# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['wxconv', 'wxconv.tests']

package_data = \
{'': ['*'],
 'wxconv': ['mapping/*'],
 'wxconv.tests': ['conll/*', 'plain_text/*', 'ssf/*', 'tnt/*']}

install_requires = \
['pbr>=2.0,<3.0', 'six>=1.12,<2.0']

entry_points = \
{'console_scripts': ['wxconv = wxconv:main']}

setup_kwargs = {
    'name': 'wxconv',
    'version': '1.0.0.0',
    'description': 'Python library for UTF to WX conversion and vice-versa for Indian languages.',
    'long_description': 'indic-wx-converter\n==================\n\n|Build Status| |Coverage Status| |CircleCI|\n\n.. |Build Status| image:: https://travis-ci.org/irshadbhat/indic-wx-converter.svg?branch=master \n   :target: https://travis-ci.org/irshadbhat/indic-wx-converter\n\n.. |Coverage Status| image:: https://coveralls.io/repos/github/irshadbhat/indic-wx-converter/badge.svg?branch=master \n   :target: https://coveralls.io/github/irshadbhat/indic-wx-converter?branch=master\n\n.. |CircleCI| image:: https://circleci.com/gh/irshadbhat/indic-wx-converter.svg?style=shield&circle-token=:circle-token \n    :target: https://circleci.com/gh/irshadbhat/indic-wx-converter\n\n\nPython library for UTF to WX conversion and vice-versa for Indian languages.\n\nInstallation\n------------\n\n::\n\n    pip install wxconv\n\nExamples\n--------\n\n1. work with files:\n^^^^^^^^^^^^^^^^^^^\n\n.. parsed-literal::\n\n    wxconv --f ssf --t intra --n --l hin --s utf --i hin-utf.ssf --o hin-wx.ssf\n\n    -l , --language     select language (3 letter ISO-639 code)\n                                        Hindi       : hin\n                                        Telugu      : tel\n                                        Tamil       : tam\n                                        Malayalam   : mal\n                                        Kannada     : kan\n                                        Bengali     : ben\n                                        Oriya       : ori\n                                        Punjabi     : pan\n                                        Marathi     : mar\n                                        Nepali      : nep\n                                        Gujarati    : guj\n                                        Bodo        : bod\n                                        Konkani     : kok\n                                        Assamese    : asm\n                                        Urdu        : urd\n    -s , --source-enc   {utf, wx} select input-file encoding\n    -f , --format       {text, ssf, conll, bio, tnt} select input-file format\n    -t , --ssf-type     {inter, intra} specify ssf-type if file format (-f) is ssf\n    -n, --nested        set this flag for nested ssf\n    -m, --no-mask       set this flag to keep off masking of roman strings in Indic text\n    -i , --input        <input-file>\n    -o , --output       <output-file>\n    -z, --normalize     set this flag for utf normalizations without WX-Conversion\n\n2. utf to wx (plain text):\n^^^^^^^^^^^^^^^^^^^^^^^^^^\n\n.. code:: python\n\n    >>> from wxconv import WXC\n    >>> \n    >>> # class WXC(order="utf2wx", format_="text", lang="hin")\n    ... # Parameters: order:str, (default="utf2wx"), source2target encoding [wx2utf|utf2wx]\n    ... #             format_:str, (default="text"), output format [text|conll|bio|tnt]\n    ... #             lang:str, (default="hin"), language parameter [hin|tel|...] (3 letter ISO-639 code)\n\n    >>> con = WXC(order=\'utf2wx\')  # here default language is hindi and default format is text\n    >>> \n    >>> hin = u"""\n    ... 1   \xe0\xa4\xa6\xe0\xa5\x87\xe0\xa4\xb6 \xe0\xa4\x95\xe0\xa5\x87 \xe0\xa4\x95\xe0\xa4\x88 \xe0\xa4\xb9\xe0\xa4\xbf\xe0\xa4\xb8\xe0\xa5\x8d\xe0\xa4\xb8\xe0\xa5\x8b\xe0\xa4\x82 \xe0\xa4\xae\xe0\xa5\x87\xe0\xa4\x82 \xe0\xa4\xb8\xe0\xa5\x82\xe0\xa4\x96\xe0\xa5\x87 \xe0\xa4\x95\xe0\xa5\x87 \xe0\xa4\x86\xe0\xa4\xb8\xe0\xa4\xbe\xe0\xa4\xb0 \xe0\xa4\x89\xe0\xa4\xa4\xe0\xa5\x8d\xe0\xa4\xaa\xe0\xa4\xa8\xe0\xa5\x8d\xe0\xa4\xa8 \xe0\xa4\xb9\xe0\xa5\x8b \xe0\xa4\x97\xe0\xa4\x8f \xe0\xa4\xb9\xe0\xa5\x88\xe0\xa4\x82\n    ... 2   \xe0\xa4\x85\xe0\xa4\xac \xe0\xa4\xa4\xe0\xa4\x95 \xe0\xa4\xae\xe0\xa5\x8c\xe0\xa4\xb8\xe0\xa4\xae \xe0\xa4\xb5\xe0\xa4\xbf\xe0\xa4\xad\xe0\xa4\xbe\xe0\xa4\x97 \xe0\xa4\xb8\xe0\xa4\xbe\xe0\xa4\xae\xe0\xa4\xbe\xe0\xa4\xa8\xe0\xa5\x8d\xe0\xa4\xaf \xe0\xa4\xac\xe0\xa4\xbe\xe0\xa4\xb0\xe0\xa4\xbf\xe0\xa4\xb6 \xe0\xa4\xb9\xe0\xa5\x8b\xe0\xa4\xa8\xe0\xa5\x87 \xe0\xa4\x95\xe0\xa5\x80 \xe0\xa4\x85\xe0\xa4\xaa\xe0\xa4\xa8\xe0\xa5\x80 \xe0\xa4\xad\xe0\xa4\xb5\xe0\xa4\xbf\xe0\xa4\xb7\xe0\xa5\x8d\xe0\xa4\xaf\xe0\xa4\xb5\xe0\xa4\xbe\xe0\xa4\xa3\xe0\xa5\x80 \xe0\xa4\xaa\xe0\xa4\xb0 \xe0\xa4\x85\xe0\xa5\x9c\xe0\xa4\xbe \xe0\xa4\xb9\xe0\xa5\x81\xe0\xa4\x86 \xe0\xa4\xa5\xe0\xa4\xbe \xe0\xa4\xb2\xe0\xa5\x87\xe0\xa4\x95\xe0\xa4\xbf\xe0\xa4\xa8 \xe0\xa4\x85\xe0\xa4\xac \xe0\xa4\xaf\xe0\xa4\xb9 \xe0\xa4\xa6\xe0\xa4\xbe\xe0\xa4\xb5\xe0\xa4\xbe \xe0\xa4\xaa\xe0\xa5\x82\xe0\xa4\xb0\xe0\xa5\x80 \xe0\xa4\xa4\xe0\xa4\xb0\xe0\xa4\xb9 \xe0\xa4\xb8\xe0\xa5\x87 \xe0\xa4\x96\xe0\xa4\xbe\xe0\xa4\xb0\xe0\xa4\xbf\xe0\xa4\x9c \xe0\xa4\xb9\xe0\xa5\x8b \xe0\xa4\x97\xe0\xa4\xaf\xe0\xa4\xbe \xe0\xa4\xb9\xe0\xa5\x88\n    ... 3   \xe0\xa4\xa6\xe0\xa5\x87\xe0\xa4\xb6 \xe0\xa4\xad\xe0\xa4\xb0 \xe0\xa4\xae\xe0\xa5\x87\xe0\xa4\x82 \xe0\xa4\x85\xe0\xa4\xac \xe0\xa4\xa4\xe0\xa4\x95 \xe0\xa4\xb9\xe0\xa5\x81\xe0\xa4\x88 \xe0\xa4\xac\xe0\xa4\xbe\xe0\xa4\xb0\xe0\xa4\xbf\xe0\xa4\xb6 \xe0\xa4\x94\xe0\xa4\xb8\xe0\xa4\xa4 \xe0\xa4\xb8\xe0\xa5\x87 \xe0\xa4\x9b\xe0\xa4\xb9 \xe0\xa4\xab\xe0\xa5\x80\xe0\xa4\xb8\xe0\xa4\xa6\xe0\xa5\x80 \xe0\xa4\x95\xe0\xa4\xae \xe0\xa4\xb9\xe0\xa5\x88 \xe0\xa4\x9c\xe0\xa4\xac\xe0\xa4\x95\xe0\xa4\xbf \xe0\xa4\xb5\xe0\xa4\xbf\xe0\xa4\xad\xe0\xa4\xbe\xe0\xa4\x97 \xe0\xa4\x95\xe0\xa4\xbe \xe0\xa4\xa6\xe0\xa4\xbe\xe0\xa4\xb5\xe0\xa4\xbe \xe0\xa4\xa5\xe0\xa4\xbe \xe0\xa4\x95\xe0\xa4\xbf \xe0\xa4\x87\xe0\xa4\xb8\xe0\xa4\xae\xe0\xa5\x87\xe0\xa4\x82 \xe0\xa5\xab \xe0\xa4\xab\xe0\xa5\x80\xe0\xa4\xb8\xe0\xa4\xa6\xe0\xa5\x80 \xe0\xa4\xb8\xe0\xa5\x87 \xe0\xa4\x9c\xe0\xa5\x8d\xe0\xa4\xaf\xe0\xa4\xbe\xe0\xa4\xa6\xe0\xa4\xbe \xe0\xa4\x95\xe0\xa4\xae\xe0\xa5\x80 \xe0\xa4\xa8\xe0\xa4\xb9\xe0\xa5\x80\xe0\xa4\x82 \xe0\xa4\xb9\xe0\xa5\x8b\xe0\xa4\x97\xe0\xa5\x80\n    ... 4   \xe0\xa4\x87\xe0\xa4\xb8\xe0\xa4\x95\xe0\xa5\x87 \xe0\xa4\x9a\xe0\xa4\xb2\xe0\xa4\xa4\xe0\xa5\x87 \xe0\xa4\x89\xe0\xa4\xa4\xe0\xa5\x8d\xe0\xa4\xa4\xe0\xa4\xb0 \xe0\xa4\xaa\xe0\xa5\x8d\xe0\xa4\xb0\xe0\xa4\xa6\xe0\xa5\x87\xe0\xa4\xb6 \xe0\xa4\xaa\xe0\xa4\x82\xe0\xa4\x9c\xe0\xa4\xbe\xe0\xa4\xac \xe0\xa4\xb9\xe0\xa4\xb0\xe0\xa4\xbf\xe0\xa4\xaf\xe0\xa4\xbe\xe0\xa4\xa3\xe0\xa4\xbe \xe0\xa4\xb0\xe0\xa4\xbe\xe0\xa4\x9c\xe0\xa4\xb8\xe0\xa5\x8d\xe0\xa4\xa5\xe0\xa4\xbe\xe0\xa4\xa8 \xe0\xa4\xac\xe0\xa4\xbf\xe0\xa4\xb9\xe0\xa4\xbe\xe0\xa4\xb0 \xe0\xa4\x9d\xe0\xa4\xbe\xe0\xa4\xb0\xe0\xa4\x96\xe0\xa4\x82\xe0\xa4\xa1 \xe0\xa4\x86\xe0\xa4\xa6\xe0\xa4\xbf \xe0\xa4\xb0\xe0\xa4\xbe\xe0\xa4\x9c\xe0\xa5\x8d\xe0\xa4\xaf \xe0\xa4\xb2\xe0\xa4\x97\xe0\xa4\xad\xe0\xa4\x97 \xe0\xa4\xb8\xe0\xa5\x82\xe0\xa4\x96\xe0\xa5\x87 \xe0\xa4\x95\xe0\xa5\x80 \xe0\xa4\x9a\xe0\xa4\xaa\xe0\xa5\x87\xe0\xa4\x9f \xe0\xa4\xae\xe0\xa5\x87\xe0\xa4\x82 \xe0\xa4\xb9\xe0\xa5\x88\xe0\xa4\x82\n    ... 5   \xe0\xa4\xb2\xe0\xa5\x87\xe0\xa4\x95\xe0\xa4\xbf\xe0\xa4\xa8 \xe0\xa4\xa4\xe0\xa4\x95\xe0\xa4\xa8\xe0\xa5\x80\xe0\xa4\x95\xe0\xa5\x80 \xe0\xa4\x95\xe0\xa4\xbe\xe0\xa4\xb0\xe0\xa4\xa3\xe0\xa5\x8b\xe0\xa4\x82 \xe0\xa4\xb8\xe0\xa5\x87 \xe0\xa4\x87\xe0\xa4\xa8\xe0\xa5\x8d\xe0\xa4\xb9\xe0\xa5\x87\xe0\xa4\x82 \xe0\xa4\x85\xe0\xa4\xad\xe0\xa5\x80 \xe0\xa4\xb8\xe0\xa5\x82\xe0\xa4\x96\xe0\xa4\xbe\xe0\xa4\x97\xe0\xa5\x8d\xe0\xa4\xb0\xe0\xa4\xb8\xe0\xa5\x8d\xe0\xa4\xa4 \xe0\xa4\x98\xe0\xa5\x8b\xe0\xa4\xb7\xe0\xa4\xbf\xe0\xa4\xa4 \xe0\xa4\xa8\xe0\xa4\xb9\xe0\xa5\x80\xe0\xa4\x82 \xe0\xa4\x95\xe0\xa4\xbf\xe0\xa4\xaf\xe0\xa4\xbe \xe0\xa4\x97\xe0\xa4\xaf\xe0\xa4\xbe \xe0\xa4\xb9\xe0\xa5\x88\n    ... """\n    >>>\n    >>> print(con.convert(hin))\n    \n    1   xeSa ke kaI hissoM meM sUKe ke AsAra uwpanna ho gae hEM\n    2   aba waka mOsama viBAga sAmAnya bAriSa hone kI apanI BaviRyavANI para adZA huA WA lekina aba yaha xAvA pUrI waraha se KArija ho gayA hE\n    3   xeSa Bara meM aba waka huI bAriSa Osawa se Caha PIsaxI kama hE jabaki viBAga kA xAvA WA ki isameM 5 PIsaxI se jyAxA kamI nahIM hogI\n    4   isake calawe uwwara praxeSa paMjAba hariyANA rAjasWAna bihAra JAraKaMda Axi rAjya lagaBaga sUKe kI capeta meM hEM\n    5   lekina wakanIkI kAraNoM se inheM aBI sUKAgraswa GoRiwa nahIM kiyA gayA hE\n    \n    >>> tel = u"""\n    ... 1   \xe0\xb0\xa4\xe0\xb0\xae 35 \xe0\xb0\x8f\xe0\xb0\xb3\xe0\xb1\x8d\xe0\xb0\xb2 \xe0\xb0\xaa\xe0\xb1\x86\xe0\xb0\xb3\xe0\xb1\x8d\xe0\xb0\xb2\xe0\xb0\xbf \xe0\xb0\xb8\xe0\xb0\x82\xe0\xb0\xa6\xe0\xb0\xb0\xe0\xb1\x8d\xe0\xb0\xad\xe0\xb0\x82\xe0\xb0\xb2\xe0\xb1\x8b\xe0\xb0\xa8\xe0\xb1\x82 - \xe0\xb0\x85\xe0\xb0\xa8\xe0\xb1\x81\xe0\xb0\x95\xe0\xb1\x8d\xe0\xb0\xb7\xe0\xb0\xa3\xe0\xb0\x82 \xe0\xb0\xb6\xe0\xb0\xa4\xe0\xb1\x8d\xe0\xb0\xb0\xe0\xb1\x81\xe0\xb0\xb5\xe0\xb1\x81\xe0\xb0\xb2\xe0\xb0\x95\xe0\xb1\x81 \xe0\xb0\xae\xe0\xb0\xb2\xe0\xb1\x8d\xe0\xb0\xb2\xe0\xb1\x87 \xe0\xb0\x95\xe0\xb0\xbe\xe0\xb0\x9f\xe0\xb1\x8d\xe0\xb0\xb2\xe0\xb0\xbe\xe0\xb0\xa1\xe0\xb1\x81\xe0\xb0\x95\xe0\xb1\x8b\xe0\xb0\xb5\xe0\xb0\x9f\xe0\xb0\x82 \xe0\xb0\xac\xe0\xb0\x82\xe0\xb0\x9f\xe0\xb1\x80\xe0\xb0\x95\xe0\xb0\xbf \xe0\xb0\xa8\xe0\xb0\x9a\xe0\xb1\x8d\xe0\xb0\x9a\xe0\xb0\xa6\xe0\xb1\x81.\n    ... 2   2007\xe0\xb0\xb2\xe0\xb1\x8b \xe0\xb0\x85\xe0\xb0\x9c\xe0\xb1\x8d\xe0\xb0\xae\xe0\xb1\x80\xe0\xb0\xb0\xe0\xb1\x8d\xe2\x80\x8c\xe0\xb0\xb2\xe0\xb1\x8b\xe0\xb0\xa8\xe0\xb0\xbf \xe0\xb0\x96\xe0\xb0\xbe\xe0\xb0\x9c\xe0\xb0\xbe \xe0\xb0\xae\xe0\xb1\x8a\xe0\xb0\xaf\xe0\xb0\xbf\xe0\xb0\xa8\xe0\xb1\x81\xe0\xb0\xa6\xe0\xb1\x8d\xe0\xb0\xa6\xe0\xb1\x80\xe0\xb0\xa8\xe0\xb1\x8d \xe0\xb0\x9a\xe0\xb0\xbf\xe0\xb0\xb7\xe0\xb1\x8d\xe0\xb0\xa4\xe0\xb1\x80 \xe0\xb0\xa6\xe0\xb0\xb0\xe0\xb1\x8d\xe0\xb0\x97\xe0\xb0\xbe\xe0\xb0\xb2\xe0\xb1\x8b \xe0\xb0\x9c\xe0\xb0\xb0\xe0\xb0\xbf\xe0\xb0\x97\xe0\xb0\xbf\xe0\xb0\xa8 \xe0\xb0\xaa\xe0\xb1\x87\xe0\xb0\xb2\xe0\xb1\x81\xe0\xb0\xa1\xe0\xb1\x81\xe0\xb0\xb2\xe0\xb1\x8b \xe0\xb0\xae\xe0\xb1\x81\xe0\xb0\x97\xe0\xb1\x8d\xe0\xb0\x97\xe0\xb1\x81\xe0\xb0\xb0\xe0\xb1\x81 \xe0\xb0\x9a\xe0\xb0\xa8\xe0\xb0\xbf\xe0\xb0\xaa\xe0\xb1\x8b\xe0\xb0\x97\xe0\xb0\xbe, \xe0\xb0\xae\xe0\xb0\xb0\xe0\xb1\x8b 15 \xe0\xb0\xae\xe0\xb0\x82\xe0\xb0\xa6\xe0\xb0\xbf \xe0\xb0\x97\xe0\xb0\xbe\xe0\xb0\xaf\xe0\xb0\xaa\xe0\xb0\xa1\xe0\xb0\xbf\xe0\xb0\xa8 \xe0\xb0\xb5\xe0\xb0\xbf\xe0\xb0\xb7\xe0\xb0\xaf\xe0\xb0\x82 \xe0\xb0\xa4\xe0\xb1\x86\xe0\xb0\xb2\xe0\xb0\xbf\xe0\xb0\xb8\xe0\xb0\xbf\xe0\xb0\x82\xe0\xb0\xa6\xe0\xb1\x87.\n    ... 3   \xe0\xb0\x97\xe0\xb0\xa3\xe0\xb0\xbf\xe0\xb0\xa4\xe0\xb0\x82 \xe0\xb0\xb2\xe0\xb0\xbe\xe0\xb0\x82\xe0\xb0\x9f\xe0\xb0\xbf \xe0\xb0\xb5\xe0\xb0\xbf\xe0\xb0\xb7\xe0\xb0\xaf\xe0\xb0\x82 \xe0\xb0\x97\xe0\xb1\x81\xe0\xb0\xb0\xe0\xb0\xbf\xe0\xb0\x82\xe0\xb0\x9a\xe0\xb0\xbf \xe0\xb0\xac\xe0\xb1\x8d\xe0\xb0\xb2\xe0\xb0\xbe\xe0\xb0\x97\xe0\xb1\x81\xe0\xb0\xb2\xe0\xb1\x81, \xe0\xb0\xb5\xe0\xb1\x80\xe0\xb0\x95\xe0\xb1\x87\xe0\xb0\xb2 \xe0\xb0\xa6\xe0\xb1\x8d\xe0\xb0\xb5\xe0\xb0\xbe\xe0\xb0\xb0\xe0\xb0\xbe \xe0\xb0\x9a\xe0\xb0\xb0\xe0\xb1\x8d\xe0\xb0\x9a\xe0\xb0\xb2\xe0\xb1\x81 \xe0\xb0\x9c\xe0\xb0\xb0\xe0\xb0\x97\xe0\xb0\xb5\xe0\xb0\x9a\xe0\xb1\x8d\xe0\xb0\x9a\xe0\xb1\x81\xe0\xb0\xa8\xe0\xb0\xa8\xe0\xb0\xbf \xe0\xb0\x85\xe0\xb0\x82\xe0\xb0\xa6\xe0\xb0\xb0\xe0\xb0\xbf\xe0\xb0\x95\xe0\xb1\x80 \xe0\xb0\x85\xe0\xb0\xb0\xe0\xb1\x8d\xe0\xb0\xa5\xe0\xb0\xae\xe0\xb0\xaf\xe0\xb0\xbf\xe0\xb0\x82\xe0\xb0\xa6\xe0\xb0\xbf.\n    ... 4   \xe0\xb0\x87\xe0\xb0\xb2\xe0\xb0\xbe \xe0\xb0\x87\xe0\xb0\x82\xe0\xb0\xa4 \xe0\xb0\xaa\xe0\xb1\x8b\xe0\xb0\x9f\xe0\xb1\x80\xe0\xb0\xb2\xe0\xb1\x8b, \xe0\xb0\xaf\xe0\xb0\xbe\xe0\xb0\x82\xe0\xb0\x9f\xe0\xb1\x80 \xe0\xb0\xac\xe0\xb1\x8d\xe0\xb0\xb0\xe0\xb0\xbf\xe0\xb0\x9f\xe0\xb0\xbf\xe0\xb0\xb7\xe0\xb1\x8d \xe0\xb0\xb5\xe0\xb0\xbe\xe0\xb0\xa4\xe0\xb0\xbe\xe0\xb0\xb5\xe0\xb0\xb0\xe0\xb0\xa3\xe0\xb0\x82\xe0\xb0\xb2\xe0\xb1\x8b\xe0\xb0\xa8\xe0\xb1\x81 155 \xe0\xb0\xa8\xe0\xb0\xbf\xe0\xb0\xae\xe0\xb1\x81\xe0\xb0\xb7\xe0\xb0\xbe\xe0\xb0\xb2 \xe0\xb0\xa8\xe0\xb0\xbf\xe0\xb0\xa1\xe0\xb0\xbf\xe0\xb0\xb5\xe0\xb0\xbf\xe0\xb0\x97\xe0\xb0\xb2 \xe2\x80\x98\xe0\xb0\xb9\xe0\xb0\xbe\xe0\xb0\xae\xe0\xb1\x8d\xe0\xb0\xb2\xe0\xb1\x86\xe0\xb0\x9f\xe0\xb1\x8d\xe2\x80\x99 \xe0\xb0\xa8\xe0\xb0\xbf\xe0\xb0\xb2\xe0\xb0\xbf\xe0\xb0\x9a\xe0\xb0\xbf \xe0\xb0\x97\xe0\xb1\x86\xe0\xb0\xb2\xe0\xb0\xbf\xe0\xb0\x9a\xe0\xb0\xbf\xe0\xb0\x82\xe0\xb0\xa6\xe0\xb0\x82\xe0\xb0\x9f\xe0\xb1\x87 \xe0\xb0\x85\xe0\xb0\xa6\xe0\xb0\xbf \xe0\xb0\xa6\xe0\xb0\xbe\xe0\xb0\xa8\xe0\xb0\xbf \xe0\xb0\x9a\xe0\xb0\x95\xe0\xb1\x8d\xe0\xb0\x95\xe0\xb0\xa6\xe0\xb0\xa8\xe0\xb0\xbe\xe0\xb0\xa8\xe0\xb0\xbf\xe0\xb0\x95\xe0\xb0\xbf \xe0\xb0\xa8\xe0\xb0\xbf\xe0\xb0\xa6\xe0\xb0\xb0\xe0\xb1\x8d\xe0\xb0\xb6\xe0\xb0\xa8\xe0\xb0\xae\xe0\xb1\x87!\n    ... 5   \xe0\xb0\x85\xe0\xb0\xb5\xe0\xb0\x95\xe0\xb0\xbe\xe0\xb0\xb6\xe0\xb0\x82 \xe0\xb0\xb5\xe0\xb0\x9a\xe0\xb1\x8d\xe0\xb0\x9a\xe0\xb0\xbf\xe0\xb0\xa8 \xe0\xb0\xb5\xe0\xb0\xbe\xe0\xb0\xb0\xe0\xb0\xbf\xe0\xb0\x95\xe0\xb0\xbf \xe0\xb0\x8e\xe0\xb0\x95\xe0\xb1\x8d\xe0\xb0\xb8\xe0\xb1\x8d\xe2\x80\x8c\xe0\xb0\xaa\xe0\xb1\x8b\xe0\xb0\x9c\xe0\xb0\xb0\xe0\xb1\x8d\xe2\x80\x8c\xe0\xb0\x95\xe0\xb1\x82\xe0\xb0\xa1\xe0\xb0\xbe \xe0\xb0\xb5\xe0\xb0\xb8\xe0\xb1\x8d\xe0\xb0\xa4\xe0\xb1\x81\xe0\xb0\x82\xe0\xb0\xa6\xe0\xb0\xbf.\n    ... """\n    >>> \n    >>> con = WXC(order=\'utf2wx\', lang=\'tel\')\n    >>>\n    >>> print(con.convert(tel))\n    \n    1   wama 35 elYla peVlYli saMxarBaMlonU - anukRaNaM Sawruvulaku malle kAtlAdukovataM baMtIki naccaxu.\n    2   2007lo ajmIr\xe2\x80\x8cloni KAjA moVyinuxxIn ciRwI xargAlo jarigina peludulo mugguru canipogA, maro 15 maMxi gAyapadina viRayaM weVlisiMxe.\n    3   gaNiwaM lAMti viRayaM guriMci blAgulu, vIkela xvArA carcalu jaragavaccunani aMxarikI arWamayiMxi.\n    4   ilA iMwa potIlo, yAMtI britiR vAwAvaraNaMlonu 155 nimuRAla nidivigala \xe2\x80\x98hAmleVt\xe2\x80\x99 nilici geVliciMxaMte axi xAni cakkaxanAniki nixarSaname!\n    5   avakASaM vaccina vAriki eVks\xe2\x80\x8cpojar\xe2\x80\x8ckUdA vaswuMxi.\n    \n    >>> \n\n3. wx to utf:\n^^^^^^^^^^^^^\n\n.. code:: python\n\n    >>> con = WXC(order=\'wx2utf\', lang=\'hin\')\n    >>> \n    >>> hin = """\n    ... 1   xeSa ke kaI hissoM meM sUKe ke AsAra uwpanna ho gae hEM\n    ... 2   aba waka mOsama viBAga sAmAnya bAriSa hone kI apanI BaviRyavANI para adZA huA WA lekina aba yaha xAvA pUrI waraha se KArija ho gayA hE\n    ... 3   xeSa Bara meM aba waka huI bAriSa Osawa se Caha PIsaxI kama hE jabaki viBAga kA xAvA WA ki isameM 5 PIsaxI se jyAxA kamI nahIM hogI\n    ... 4   isake calawe uwwara praxeSa paMjAba hariyANA rAjasWAna bihAra JAraKaMda Axi rAjya lagaBaga sUKe kI capeta meM hEM\n    ... 5   lekina wakanIkI kAraNoM se inheM aBI sUKAgraswa GoRiwa nahIM kiyA gayA hE\n    ... """\n    >>> \n    >>> print(con.convert(hin))\n    \n    1   \xe0\xa4\xa6\xe0\xa5\x87\xe0\xa4\xb6 \xe0\xa4\x95\xe0\xa5\x87 \xe0\xa4\x95\xe0\xa4\x88 \xe0\xa4\xb9\xe0\xa4\xbf\xe0\xa4\xb8\xe0\xa5\x8d\xe0\xa4\xb8\xe0\xa5\x8b\xe0\xa4\x82 \xe0\xa4\xae\xe0\xa5\x87\xe0\xa4\x82 \xe0\xa4\xb8\xe0\xa5\x82\xe0\xa4\x96\xe0\xa5\x87 \xe0\xa4\x95\xe0\xa5\x87 \xe0\xa4\x86\xe0\xa4\xb8\xe0\xa4\xbe\xe0\xa4\xb0 \xe0\xa4\x89\xe0\xa4\xa4\xe0\xa5\x8d\xe0\xa4\xaa\xe0\xa4\xa8\xe0\xa5\x8d\xe0\xa4\xa8 \xe0\xa4\xb9\xe0\xa5\x8b \xe0\xa4\x97\xe0\xa4\x8f \xe0\xa4\xb9\xe0\xa5\x88\xe0\xa4\x82\n    2   \xe0\xa4\x85\xe0\xa4\xac \xe0\xa4\xa4\xe0\xa4\x95 \xe0\xa4\xae\xe0\xa5\x8c\xe0\xa4\xb8\xe0\xa4\xae \xe0\xa4\xb5\xe0\xa4\xbf\xe0\xa4\xad\xe0\xa4\xbe\xe0\xa4\x97 \xe0\xa4\xb8\xe0\xa4\xbe\xe0\xa4\xae\xe0\xa4\xbe\xe0\xa4\xa8\xe0\xa5\x8d\xe0\xa4\xaf \xe0\xa4\xac\xe0\xa4\xbe\xe0\xa4\xb0\xe0\xa4\xbf\xe0\xa4\xb6 \xe0\xa4\xb9\xe0\xa5\x8b\xe0\xa4\xa8\xe0\xa5\x87 \xe0\xa4\x95\xe0\xa5\x80 \xe0\xa4\x85\xe0\xa4\xaa\xe0\xa4\xa8\xe0\xa5\x80 \xe0\xa4\xad\xe0\xa4\xb5\xe0\xa4\xbf\xe0\xa4\xb7\xe0\xa5\x8d\xe0\xa4\xaf\xe0\xa4\xb5\xe0\xa4\xbe\xe0\xa4\xa3\xe0\xa5\x80 \xe0\xa4\xaa\xe0\xa4\xb0 \xe0\xa4\x85\xe0\xa4\xa1\xe0\xa4\xbc\xe0\xa4\xbe \xe0\xa4\xb9\xe0\xa5\x81\xe0\xa4\x86 \xe0\xa4\xa5\xe0\xa4\xbe \xe0\xa4\xb2\xe0\xa5\x87\xe0\xa4\x95\xe0\xa4\xbf\xe0\xa4\xa8 \xe0\xa4\x85\xe0\xa4\xac \xe0\xa4\xaf\xe0\xa4\xb9 \xe0\xa4\xa6\xe0\xa4\xbe\xe0\xa4\xb5\xe0\xa4\xbe \xe0\xa4\xaa\xe0\xa5\x82\xe0\xa4\xb0\xe0\xa5\x80 \xe0\xa4\xa4\xe0\xa4\xb0\xe0\xa4\xb9 \xe0\xa4\xb8\xe0\xa5\x87 \xe0\xa4\x96\xe0\xa4\xbe\xe0\xa4\xb0\xe0\xa4\xbf\xe0\xa4\x9c \xe0\xa4\xb9\xe0\xa5\x8b \xe0\xa4\x97\xe0\xa4\xaf\xe0\xa4\xbe \xe0\xa4\xb9\xe0\xa5\x88\n    3   \xe0\xa4\xa6\xe0\xa5\x87\xe0\xa4\xb6 \xe0\xa4\xad\xe0\xa4\xb0 \xe0\xa4\xae\xe0\xa5\x87\xe0\xa4\x82 \xe0\xa4\x85\xe0\xa4\xac \xe0\xa4\xa4\xe0\xa4\x95 \xe0\xa4\xb9\xe0\xa5\x81\xe0\xa4\x88 \xe0\xa4\xac\xe0\xa4\xbe\xe0\xa4\xb0\xe0\xa4\xbf\xe0\xa4\xb6 \xe0\xa4\x94\xe0\xa4\xb8\xe0\xa4\xa4 \xe0\xa4\xb8\xe0\xa5\x87 \xe0\xa4\x9b\xe0\xa4\xb9 \xe0\xa4\xab\xe0\xa5\x80\xe0\xa4\xb8\xe0\xa4\xa6\xe0\xa5\x80 \xe0\xa4\x95\xe0\xa4\xae \xe0\xa4\xb9\xe0\xa5\x88 \xe0\xa4\x9c\xe0\xa4\xac\xe0\xa4\x95\xe0\xa4\xbf \xe0\xa4\xb5\xe0\xa4\xbf\xe0\xa4\xad\xe0\xa4\xbe\xe0\xa4\x97 \xe0\xa4\x95\xe0\xa4\xbe \xe0\xa4\xa6\xe0\xa4\xbe\xe0\xa4\xb5\xe0\xa4\xbe \xe0\xa4\xa5\xe0\xa4\xbe \xe0\xa4\x95\xe0\xa4\xbf \xe0\xa4\x87\xe0\xa4\xb8\xe0\xa4\xae\xe0\xa5\x87\xe0\xa4\x82 5 \xe0\xa4\xab\xe0\xa5\x80\xe0\xa4\xb8\xe0\xa4\xa6\xe0\xa5\x80 \xe0\xa4\xb8\xe0\xa5\x87 \xe0\xa4\x9c\xe0\xa5\x8d\xe0\xa4\xaf\xe0\xa4\xbe\xe0\xa4\xa6\xe0\xa4\xbe \xe0\xa4\x95\xe0\xa4\xae\xe0\xa5\x80 \xe0\xa4\xa8\xe0\xa4\xb9\xe0\xa5\x80\xe0\xa4\x82 \xe0\xa4\xb9\xe0\xa5\x8b\xe0\xa4\x97\xe0\xa5\x80\n    4   \xe0\xa4\x87\xe0\xa4\xb8\xe0\xa4\x95\xe0\xa5\x87 \xe0\xa4\x9a\xe0\xa4\xb2\xe0\xa4\xa4\xe0\xa5\x87 \xe0\xa4\x89\xe0\xa4\xa4\xe0\xa5\x8d\xe0\xa4\xa4\xe0\xa4\xb0 \xe0\xa4\xaa\xe0\xa5\x8d\xe0\xa4\xb0\xe0\xa4\xa6\xe0\xa5\x87\xe0\xa4\xb6 \xe0\xa4\xaa\xe0\xa4\x82\xe0\xa4\x9c\xe0\xa4\xbe\xe0\xa4\xac \xe0\xa4\xb9\xe0\xa4\xb0\xe0\xa4\xbf\xe0\xa4\xaf\xe0\xa4\xbe\xe0\xa4\xa3\xe0\xa4\xbe \xe0\xa4\xb0\xe0\xa4\xbe\xe0\xa4\x9c\xe0\xa4\xb8\xe0\xa5\x8d\xe0\xa4\xa5\xe0\xa4\xbe\xe0\xa4\xa8 \xe0\xa4\xac\xe0\xa4\xbf\xe0\xa4\xb9\xe0\xa4\xbe\xe0\xa4\xb0 \xe0\xa4\x9d\xe0\xa4\xbe\xe0\xa4\xb0\xe0\xa4\x96\xe0\xa4\x82\xe0\xa4\xa1 \xe0\xa4\x86\xe0\xa4\xa6\xe0\xa4\xbf \xe0\xa4\xb0\xe0\xa4\xbe\xe0\xa4\x9c\xe0\xa5\x8d\xe0\xa4\xaf \xe0\xa4\xb2\xe0\xa4\x97\xe0\xa4\xad\xe0\xa4\x97 \xe0\xa4\xb8\xe0\xa5\x82\xe0\xa4\x96\xe0\xa5\x87 \xe0\xa4\x95\xe0\xa5\x80 \xe0\xa4\x9a\xe0\xa4\xaa\xe0\xa5\x87\xe0\xa4\x9f \xe0\xa4\xae\xe0\xa5\x87\xe0\xa4\x82 \xe0\xa4\xb9\xe0\xa5\x88\xe0\xa4\x82\n    5   \xe0\xa4\xb2\xe0\xa5\x87\xe0\xa4\x95\xe0\xa4\xbf\xe0\xa4\xa8 \xe0\xa4\xa4\xe0\xa4\x95\xe0\xa4\xa8\xe0\xa5\x80\xe0\xa4\x95\xe0\xa5\x80 \xe0\xa4\x95\xe0\xa4\xbe\xe0\xa4\xb0\xe0\xa4\xa3\xe0\xa5\x8b\xe0\xa4\x82 \xe0\xa4\xb8\xe0\xa5\x87 \xe0\xa4\x87\xe0\xa4\xa8\xe0\xa5\x8d\xe0\xa4\xb9\xe0\xa5\x87\xe0\xa4\x82 \xe0\xa4\x85\xe0\xa4\xad\xe0\xa5\x80 \xe0\xa4\xb8\xe0\xa5\x82\xe0\xa4\x96\xe0\xa4\xbe\xe0\xa4\x97\xe0\xa5\x8d\xe0\xa4\xb0\xe0\xa4\xb8\xe0\xa5\x8d\xe0\xa4\xa4 \xe0\xa4\x98\xe0\xa5\x8b\xe0\xa4\xb7\xe0\xa4\xbf\xe0\xa4\xa4 \xe0\xa4\xa8\xe0\xa4\xb9\xe0\xa5\x80\xe0\xa4\x82 \xe0\xa4\x95\xe0\xa4\xbf\xe0\xa4\xaf\xe0\xa4\xbe \xe0\xa4\x97\xe0\xa4\xaf\xe0\xa4\xbe \xe0\xa4\xb9\xe0\xa5\x88\n    \n    >>> \n\n4. work with conll:\n^^^^^^^^^^^^^^^^^^^\n\n.. code:: python\n\n    >>> con = WXC(order=\'utf2wx\', lang=\'hin\', format_=\'conll\')\n    >>> \n    >>> conll = u"""\n    ... 1       \xe0\xa4\x87\xe0\xa4\xb8\xe0\xa4\x95\xe0\xa5\x80     \xe0\xa4\xaf\xe0\xa4\xb9      pn      PRP     cat-pn|gen-f|num-sg|pers-3|case-o|vib-\xe0\xa4\x95\xe0\xa4\xbe|tam-kA|chunkId-NP|chunkType-head|stype-|voicetype-      2     r6      _       _\n    ... 2       \xe0\xa4\x8a\xe0\xa4\x81\xe0\xa4\x9a\xe0\xa4\xbe\xe0\xa4\x88     \xe0\xa4\x8a\xe0\xa4\x81\xe0\xa4\x9a\xe0\xa4\xbe\xe0\xa4\x88     n       NN      cat-n|gen-f|num-sg|pers-3|case-d|vib-0|tam-0|chunkId-NP2|chunkType-head|stype-|voicetype-       6     k1      _       _\n    ... 3       \xe0\xa4\x95\xe0\xa5\x87\xe0\xa4\xb5\xe0\xa4\xb2     \xe0\xa4\x95\xe0\xa5\x87\xe0\xa4\xb5\xe0\xa4\xb2     avy     RP      cat-avy|gen-|num-|pers-|case-|vib-|tam-|chunkId-NP3|chunkType-child|stype-|voicetype-   4       lwg__rp _       _\n    ... 4       1982    1982    num     QC      cat-num|gen-any|num-any|pers-|case-any|vib-|tam-|chunkId-NP3|chunkType-child|stype-|voicetype-  5       nmod__adj       _       _\n    ... 5       \xe0\xa4\xae\xe0\xa5\x80\xe0\xa4\x9f\xe0\xa4\xb0     \xe0\xa4\xae\xe0\xa5\x80\xe0\xa4\x9f\xe0\xa4\xb0     n       NN      cat-n|gen-m|num-sg|pers-3|case-d|vib-0|tam-0|chunkId-NP3|chunkType-head|stype-|voicetype-       6     k1s     _       _\n    ... 6       \xe0\xa4\xb9\xe0\xa5\x88       \xe0\xa4\xb9\xe0\xa5\x88       v       VM      cat-v|gen-any|num-sg|pers-3|case-|vib-\xe0\xa4\xb9\xe0\xa5\x88|tam-hE|chunkId-VGF|chunkType-head|stype-declarative|voicetype-active    0       root    _       _\n    ... 7       \xe0\xa5\xa4       \xe0\xa5\xa4       punc    SYM     cat-punc|gen-|num-|pers-|case-|vib-|tam-|chunkId-BLK|chunkType-head|stype-|voicetype-   6       rsym    _       _"""\n    >>> \n    >>> print(con.convert(conll))\n    \n    1   isakI   yaha    pn      PRP cat-pn|gen-f|num-sg|pers-3|case-o|vib-kA|tam-kA|chunkId-NP|chunkType-head|stype-|voicetype- 2   r6  _   _\n    2   UzcAI   UzcAI   n       NN  cat-n|gen-f|num-sg|pers-3|case-d|vib-0|tam-0|chunkId-NP2|chunkType-head|stype-|voicetype-   6   k1  _   _\n    3   kevala  kevala  avy     RP  cat-avy|gen-|num-|pers-|case-|vib-|tam-|chunkId-NP3|chunkType-child|stype-|voicetype-   4   lwg__rp_    _\n    4   1982    1982    num     QC  cat-num|gen-any|num-any|pers-|case-any|vib-|tam-|chunkId-NP3|chunkType-child|stype-|voicetype-  5   nmod__adj   _   _\n    5   mItara  mItara  n       NN  cat-n|gen-m|num-sg|pers-3|case-d|vib-0|tam-0|chunkId-NP3|chunkType-head|stype-|voicetype-   6   k1s _   _\n    6   hE      hE  v   VM  cat-v|gen-any|num-sg|pers-3|case-|vib-hE|tam-hE|chunkId-VGF|chunkType-head|stype-declarative|voicetype-active   0   root    _   _\n    7   .       .   punc    SYM cat-punc|gen-|num-|pers-|case-|vib-|tam-|chunkId-BLK|chunkType-head|stype-|voicetype-   6   rsym    __\n    >>> \n\n5. work with tnt:\n^^^^^^^^^^^^^^^^^\n\n.. code:: python\n\n    >>> tnt = u"""\n    ... \xe0\xa4\xaf\xe0\xa5\x8b\xe0\xa4\x82       RB\n    ... \xe0\xa4\xb8\xe0\xa4\xbf\xe0\xa4\x82\xe0\xa4\x97\xe0\xa4\xb2     JJ\n    ... \xe0\xa4\xb8\xe0\xa5\x8d\xe0\xa4\x95\xe0\xa5\x8d\xe0\xa4\xb0\xe0\xa5\x80\xe0\xa4\xa8    NNC\n    ... \xe0\xa4\xa5\xe0\xa4\xbf\xe0\xa4\x8f\xe0\xa4\x9f\xe0\xa4\xb0    NNP\n    ... \xe0\xa4\x95\xe0\xa5\x87       PSP\n    ... \xe0\xa4\xa6\xe0\xa4\xb0\xe0\xa5\x8d\xe0\xa4\xb6\xe0\xa4\x95\xe0\xa5\x8b\xe0\xa4\x82    NN\n    ... \xe0\xa4\x95\xe0\xa5\x8b       PSP\n    ... \xe0\xa4\x85\xe0\xa4\x97\xe0\xa5\x8d\xe0\xa4\xa8\xe0\xa4\xbf\xe0\xa4\xaa\xe0\xa4\xa5   NNP\n    ... \xe0\xa4\x85\xe0\xa4\xa7\xe0\xa4\xbf\xe0\xa4\x95     QF\n    ... \xe0\xa4\xa8\xe0\xa4\xb9\xe0\xa5\x80\xe0\xa4\x82      NEG\n    ... \xe0\xa4\xad\xe0\xa4\xbe       VM\n    ... \xe0\xa4\xb8\xe0\xa4\x95\xe0\xa5\x80      VAUX\n    ... \xe0\xa5\xa4       SYM\n    ... """\n    >>> \n    >>> con = WXC(order=\'utf2wx\', lang=\'hin\', format_=\'tnt\')\n    >>> \n    >>> print(con.convert(tnt))\n    \n    yoM RB\n    siMgala     JJ\n    skrIna      NNC\n    Wietara     NNP\n    ke  PSP\n    xarSakoM    NN\n    ko  PSP\n    agnipaWa    NNP\n    aXika       QF\n    nahIM       NEG\n    BA  VM\n    sakI        VAUX\n    .   SYM\n    \n    >>> \n\n6. work with bio:\n^^^^^^^^^^^^^^^^^\n\n::\n\n    same as tnt or conll\n\n7. work with ssf:\n^^^^^^^^^^^^^^^^^\n\n::\n    \n    implemented for ssf files (intra, inter and nested)\n\n',
    'author': 'irshadbhat',
    'author_email': 'bhatirshad127@gmail.com',
    'url': 'https://github.com/irshadbhat/indic-wx-converter',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
}


setup(**setup_kwargs)
